@@ -62,6 +62,7 @@ class SimplePOP3Retriever(POP3RetrieverBase, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
     )
@@ -95,10 +96,16 @@ class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
         ConfInt(name='port', required=False, default=POP3_ssl_port),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
+        ConfFile(name='ca_certs', required=False, default=None),
+        ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
+        ConfString(name='ssl_version', required=False, default=None),
+        ConfString(name='ssl_ciphers', required=False, default=None),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'POP3-SSL'
@@ -134,7 +141,7 @@ class BrokenUIDLPOP3RetrieverBase(POP3RetrieverBase):
         duplicated IDs are always treated as new messages.'''
         self.log.trace()
 
-    def write_oldmailfile(self, **kwargs):
+    def write_oldmailfile(self, unused, **kwargs):
         '''Short-circuit writing the oldmail file.'''
         self.log.trace()
 
@@ -167,6 +174,7 @@ class BrokenUIDLPOP3Retriever(BrokenUIDLPOP3RetrieverBase, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
     )
     received_with = 'POP3'
@@ -197,9 +205,15 @@ class BrokenUIDLPOP3SSLRetriever(BrokenUIDLPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfInt(name='port', required=False, default=POP3_ssl_port),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
+        ConfFile(name='ca_certs', required=False, default=None),
+        ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
+        ConfString(name='ssl_version', required=False, default=None),
+        ConfString(name='ssl_ciphers', required=False, default=None),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_with = 'POP3-SSL'
 
@@ -229,6 +243,7 @@ class MultidropPOP3Retriever(MultidropPOP3RetrieverBase, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
     )
@@ -262,10 +277,16 @@ class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfInt(name='port', required=False, default=POP3_ssl_port),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
+        ConfFile(name='ca_certs', required=False, default=None),
+        ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
+        ConfString(name='ssl_version', required=False, default=None),
+        ConfString(name='ssl_ciphers', required=False, default=None),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'POP3-SSL'
@@ -304,6 +325,7 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
         ConfInt(name='port', required=False, default=110),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
         # Demon apparently doesn't support APOP
         ConfBool(name='use_apop', required=False, default=False),
     )
@@ -356,13 +378,17 @@ class SimpleIMAPRetriever(IMAPRetrieverBase, IMAPinitMixIn):
         ConfInt(name='port', required=False, default=imaplib.IMAP4_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
-        ConfTupleOfStrings(name='mailboxes', required=False,
-                           default="('INBOX', )"),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfTupleOfUnicode(name='mailboxes', required=False,
+                           default="('INBOX', )", allow_specials=('ALL',)),
+        ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfBool(name='record_mailbox', required=False, default=True),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfBool(name='use_xoauth2', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4'
@@ -389,21 +415,31 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInstance(name='configparser', required=False),
         ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        # socket.ssl() and socket timeouts are incompatible in Python 2.3
-        #ConfInt(name='timeout', required=False, default=180),
+        # socket.ssl() and socket timeouts were incompatible in Python 2.3;
+        # if you have problems, comment this line out
+        ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
-        ConfTupleOfStrings(name='mailboxes', required=False,
-                           default="('INBOX', )"),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfTupleOfUnicode(name='mailboxes', required=False,
+                           default="('INBOX', )", allow_specials=('ALL',)),
+        ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfBool(name='record_mailbox', required=False, default=True),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
+        ConfFile(name='ca_certs', required=False, default=None),
+        ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
+        ConfString(name='ssl_version', required=False, default=None),
+        ConfString(name='ssl_ciphers', required=False, default=None),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfBool(name='use_xoauth2', required=False, default=False),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -435,13 +471,17 @@ class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
         ConfInt(name='port', required=False, default=imaplib.IMAP4_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
-        ConfTupleOfStrings(name='mailboxes', required=False,
-                           default="('INBOX', )"),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfTupleOfUnicode(name='mailboxes', required=False,
+                           default="('INBOX', )", allow_specials=('ALL',)),
+        ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfBool(name='record_mailbox', required=False, default=True),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfString(name='envelope_recipient'),
     )
     received_from = None
@@ -469,22 +509,32 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInstance(name='configparser', required=False),
         ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        # socket.ssl() and socket timeouts are incompatible in Python 2.3
-        #ConfInt(name='timeout', required=False, default=180),
+        # socket.ssl() and socket timeouts were incompatible in Python 2.3;
+        # if you have problems, comment this line out
+        ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
-        ConfTupleOfStrings(name='mailboxes', required=False,
-                           default="('INBOX', )"),
+        ConfTupleOfStrings(name='password_command', required=False, default=()),
+        ConfTupleOfUnicode(name='mailboxes', required=False,
+                           default="('INBOX', )", allow_specials=('ALL',)),
+        ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
+        ConfBool(name='record_mailbox', required=False, default=True),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
+        ConfFile(name='ca_certs', required=False, default=None),
+        ConfTupleOfStrings(name='ssl_fingerprints', required=False, default=()),
+        ConfString(name='ssl_version', required=False, default=None),
+        ConfString(name='ssl_ciphers', required=False, default=None),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfString(name='envelope_recipient'),
+        ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -502,3 +552,4 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         self.log.trace()
         self.log.info('MultidropIMAPSSLRetriever(%s)' % self._confstring()
                       + os.linesep)
+
